# lab5.py


# For complete documentation see:
# https://developer.leapmotion.com/documentation/v2/python/index.html

# common import
import sys
sys.path.append('..')
from common.core import *
from common.gfxutil import *
from common.leaputil import *
from common.synth import *
from common.audio import *

import numpy as np
import Leap


# Create a "one-handed Leap Theramin" according to this spec:
#
#  a. When the hand breaks through the XY plane (at some value of z), a note will play.
#     The pitch of the note is determined by the height of the hand, and mapped to a 
#     1-octave c-major scale. The note turns off when the hand is pulled out of the XY plane.
#     Also, choose a sustaining sound - like a string, wind, or brass instrument.
#
#  b. Modify the system so as the hand moves up and down, the sounding note changes according
#     to the same mapping of height to c-major scale.
#
#  c. Add volume control, mapping the x position of the hand to volume. Recall that volume is 
#     sent as: synth.cc(channel, 7, v) where v = [0, 127]


class MainWidget1(BaseWidget) :
    def __init__(self):
        super(MainWidget1, self).__init__()

        self.audio = Audio(2)
        self.synth = Synth("../data/FluidR3_GM.sf2")
        self.audio.set_generator(self.synth)

        self.leap = Leap.Controller()

        self.hand_disp = Cursor3D((Window.width/2,Window.height/2), (10, 10), (.2, .6, .2))
        self.canvas.add(self.hand_disp)

        self.label = topleft_label()
        self.add_widget(self.label)


    def on_update(self) :
        self.audio.on_update()

        leap_frame = self.leap.frame()
        pt = leap_one_palm(leap_frame)

        norm_pt = scale_point(pt)

        self.hand_disp.set_pos(norm_pt)

        self.label.text = leap_info(self.leap)
        self.label.text += 'x:%d\ny:%d\nz:%d\n' % (pt[0], pt[1], pt[2])
        self.label.text += 'x:%.2f\ny:%.2f\nz:%.2f\n' % (norm_pt[0], norm_pt[1], norm_pt[2])


# convert the 3D point pt into unit scale (ie, range [0,1]) assuming that pt falls in the
# the range [RANGE_MIN, RANGE_MAX]. Value is clipped [0,1]
RANGE_MIN = np.array((-250.0, 50, -200))
RANGE_MAX = np.array((250.0, 500, 200))
def scale_point(pt):
    pt = (pt - RANGE_MIN) / (RANGE_MAX - RANGE_MIN)
    pt = np.clip(pt, 0, 1)
    return pt


run(MainWidget1)
